1#ifndef __MATH_MATH_KERNEL_H__
2#define __MATH_MATH_KERNEL_H__
48 for (
int i = idx;
i < n;
i +=
str) {
92 const int ly,
const int lz) {
93 const int idx2 = idx - 1;
94 index[3] =
idx2 / (lx * ly * lz);
95 index[2] = (
idx2 - (lx * ly * lz) * index[3]) / (lx * ly);
96 index[1] = (
idx2 - (lx * ly * lz) * index[3] - (lx * ly) * index[2]) / lx;
97 index[0] = (
idx2 - (lx * ly * lz) * index[3] - (lx * ly) * index[2]) -
107 const int n1,
const int n2,
const int nf) {
108 return ((
i) + (n1) * (((
j) - 1) + (n2) * (((
k) - 1) + (
nf) * (((l) - 1))))) - 1;
114template<
typename T >
130 for (
int m = idx; m <
n_mask; m +=
str) {
131 const int f = facet[m + 1];
155template<
typename T >
173template<
typename T >
188#if __CUDA_ARCH__ < 600
195template<
typename T >
205#if __CUDA_ARCH__ >= 600
206 for (
int i = idx;
i < m;
i +=
str)
210 assert(0 &&
"masked_atomic_reduction_kernel requires compute capability 6.0 or higher.");
217template<
typename T >
235template<
typename T >
258 const int mask_size) {
263 for (
int i = idx;
i < mask_size;
i +=
str) {
a[
mask[
i]] = c; }
269template<
typename T >
278 for (
int i = idx;
i < n;
i +=
str) {
286template<
typename T >
294 for (
int i = idx;
i < n;
i +=
str) {
302template<
typename T >
311 for (
int i = idx;
i < n;
i +=
str) {
319template<
typename T >
327 for (
int i = idx;
i < n;
i +=
str) {
335template<
typename T >
344 for (
int i = idx;
i < n;
i +=
str) {
352template<
typename T >
362 for (
int i = idx;
i < n;
i +=
str) {
370template<
typename T >
378 for (
int i = idx;
i < n;
i +=
str) {
386template<
typename T >
394 for (
int i = idx;
i < n;
i +=
str) {
402template<
typename T >
411 for (
int i = idx;
i < n;
i +=
str) {
419template<
typename T >
429 for (
int i = idx;
i < n;
i +=
str) {
430 a[
i] = b[
i] + c[
i] + d[
i];
437template<
typename T >
446 for (
int i = idx;
i < n;
i +=
str) {
454template<
typename T >
465 for (
int i = idx;
i < n;
i+=
str) {
468 tmp += p[
j][
i]*alpha[
j];
477template<
typename T >
486 for (
int i = idx;
i < n;
i +=
str) {
494template<
typename T >
503 for (
int i = idx;
i < n;
i +=
str) {
511template<
typename T >
522 for (
int i = idx;
i < n;
i +=
str) {
530template<
typename T >
543 for (
int i = idx;
i < n;
i +=
str) {
551template<
typename T >
566 for (
int i = idx;
i < n;
i +=
str) {
574template<
typename T >
582 for (
int i = idx;
i < n;
i +=
str) {
590template<
typename T >
598 for (
int i = idx;
i < n;
i +=
str) {
606template<
typename T >
615 for (
int i = idx;
i < n;
i +=
str) {
623template<
typename T >
631 for (
int i = idx;
i < n;
i +=
str) {
639template<
typename T >
648 for (
int i = idx;
i < n;
i +=
str) {
656template<
typename T >
665 for (
int i = idx;
i < n;
i +=
str) {
666 a[
i] =
a[
i] - b[
i] * c[
i];
673template<
typename T >
681 for (
int i = idx;
i < n;
i +=
str) {
689template<
typename T >
698 for (
int i = idx;
i < n;
i +=
str) {
706template<
typename T >
715 for (
int i = idx;
i < n;
i +=
str) {
716 a[
i] =
a[
i] + b[
i] * c[
i];
724template<
typename T >
734 for (
int i = idx;
i < n;
i +=
str) {
735 a[
i] =
a[
i] + b[
i] * c[
i] * d[
i];
743template<
typename T >
753 for (
int i = idx;
i < n;
i +=
str) {
754 a[
i] =
a[
i] + s * b[
i] * c[
i];
762template<
typename T >
775 for (
int i = idx;
i < n;
i +=
str) {
784template<
typename T >
799 for (
int i = idx;
i < n;
i +=
str) {
802 u3[
i] = v1[
i]*w2[
i] - v2[
i]*w1[
i];
850template<
typename T >
856 for (
int i = idx;
i<n ;
i +=
str)
881template<
typename T >
887 for (
int i = idx;
i<n ;
i +=
str)
912template<
typename T >
918 for (
int i = idx;
i<n ;
i +=
str)
944template<
typename T >
955 for (
int i=idx ;
i<n ;
i+=step)
979template<
typename T >
994 for (
int i = idx;
i < n;
i+=
str) {
1014template<
typename T >
1029 for (
int i = idx;
i < n;
i+=
str) {
1055template<
typename T >
1069 for (
int i = idx;
i < n;
i+=
str) {
1090template<
typename T >
1104 for (
int i = idx;
i < n;
i+=
str) {
1125template<
typename T >
1138 for (
int i = idx;
i<n ;
i +=
str)
1160template<
typename T >
1174 for (
int i = idx;
i<n ;
i +=
str)
1196template<
typename T >
1210 for (
int i = idx;
i<n ;
i +=
str)
1232template<
typename T >
1239 for (
int i = idx;
i < n;
i +=
str) {
1251template <
typename T>
1265template <
typename T>
1280template <
typename T>
1293template <
typename T>
1300 for (
int i = idx;
i < n;
i +=
str)
a[
i] =
max(b[
i], c);
1307template <
typename T>
1321template <
typename T>
1336template <
typename T>
1349template <
typename T>
1356 for (
int i = idx;
i < n;
i +=
str)
a[
i] =
min(b[
i], c);
__global__ void ale_add_kinematics_kernel(const int n, T *__restrict__ wx, T *__restrict__ wy, T *__restrict__ wz, const T *__restrict__ x_ref, const T *__restrict__ y_ref, const T *__restrict__ z_ref, const T *__restrict__ phi, const T *__restrict__ x, const T *__restrict__ y, const T *__restrict__ z, const kinematics_params_t kin_params)
__global__ void const T *__restrict__ x
__global__ void const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ w3
__global__ void addcol4_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const int n)
__global__ void cwrap_kernel(T *__restrict__ a, const T min_val, const T max_val, const int n)
__global__ void pwmin_vec3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void reduce_kernel(T *bufred, const int n)
__global__ void cdiv2_kernel(T *__restrict__ a, T *__restrict__ b, const T c, const int n)
__global__ void invcol2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void add2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void add4s3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const T c1, const T c2, const T c3, const int n)
__inline__ __device__ T reduce_warp(T val)
__global__ void masked_scatter_copy_aligned_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void masked_atomic_reduction_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int m)
__global__ void pwmax_vec3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void glsc3_many_kernel(const T *a, const T **b, const T *c, T *buf_h, const int j, const int n)
__global__ void addcol3s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T s, const int n)
__global__ void cfill_mask_kernel(T *__restrict__ a, const T c, const int size, int *__restrict__ mask, const int mask_size)
__global__ void cdiv_kernel(T *__restrict__ a, const T c, const int n)
__global__ void masked_copy_kernel_aligned(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void face_masked_gather_copy_kernel(T *__restrict__ a, const T *__restrict__ b, const int *__restrict__ mask, const int *__restrict__ facet, const int n1, const int n2, const int lx, const int ly, const int lz, const int n_mask)
__global__ void glsc3_reduce_kernel(T *bufred, const int n, const int j)
__global__ void masked_gather_copy_aligned_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void pwmax_sca2_kernel(T *__restrict__ a, const T c, const int n)
__device__ __forceinline__ void face_gather_nonlinear_index(int *index, const int idx, const int lx, const int ly, const int lz)
__global__ void reduce_max_kernel(T *bufred, const T ninf, const int n)
__global__ void glsubnorm2_kernel(const T *a, const T *b, T *buf_h, const int n)
__global__ void pwmin_vec2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void add3s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T c1, const T c2, const int n)
__inline__ __device__ T reduce_max_warp(T val)
__global__ void add2s1_kernel(T *__restrict__ a, const T *__restrict__ b, const T c1, const int n)
__device__ __forceinline__ int face_gather_idx(const int i, const int j, const int k, const int l, const int n1, const int n2, const int nf)
__global__ void add5s4_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const T *__restrict__ e, const T c1, const T c2, const T c3, const T c4, const int n)
__global__ void masked_gather_copy_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void add2s2_many_kernel(T *__restrict__ x, const T **p, const T *alpha, const int p_cur, const int n)
__global__ void pwmax_vec2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void cmult_kernel(T *__restrict__ a, const T c, const int n)
__global__ void addcol3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void pwmin_sca3_kernel(T *__restrict__ a, const T *__restrict b, const T c, const int n)
__global__ void pwmax_sca3_kernel(T *__restrict__ a, const T *__restrict b, const T c, const int n)
__global__ void col2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void masked_copy_kernel_0(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void col3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void sub2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void glmin_kernel(const T *a, const T pinf, T *buf_h, const int n)
__global__ void glsc2_kernel(const T *a, const T *b, T *buf_h, const int n)
__global__ void cmult2_kernel(T *__restrict__ a, T *__restrict__ b, const T c, const int n)
__global__ void pwmin_sca2_kernel(T *__restrict__ a, const T c, const int n)
__global__ void sub3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void glsum_kernel(const T *a, T *buf_h, const int n)
__global__ void glsc3_kernel(const T *a, const T *b, const T *c, T *buf_h, const int n)
__global__ void add2s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T c1, const int n)
__global__ void vdot3_kernel(T *__restrict__ dot, const T *__restrict__ u1, const T *__restrict__ u2, const T *__restrict__ u3, const T *__restrict__ v1, const T *__restrict__ v2, const T *__restrict__ v3, const int n)
__global__ void invcol1_kernel(T *__restrict__ a, const int n)
__global__ void add3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void glmax_kernel(const T *a, const T ninf, T *buf_h, const int n)
__global__ void add4_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const int n)
__global__ void cfill_kernel(T *__restrict__ a, const T c, const int n)
__global__ void masked_scatter_copy_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void reduce_min_kernel(T *bufred, const T pinf, const int n)
__global__ void vcross_kernel(T *__restrict__ u1, T *__restrict__ u2, T *__restrict__ u3, const T *__restrict__ v1, const T *__restrict__ v2, const T *__restrict__ v3, const T *__restrict__ w1, const T *__restrict__ w2, const T *__restrict__ w3, const int n)
__global__ void addsqr2s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T c1, const int n)
__global__ void cadd2_kernel(T *__restrict__ a, T *__restrict__ b, const T c, const int n)
__global__ void absval_kernel(T *__restrict__ a, const int n)
__global__ void invcol3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void subcol3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void cadd_kernel(T *__restrict__ a, const T c, const int n)
__inline__ __device__ T reduce_min_warp(T val)
Object for handling masks in Neko.